#!/usr/bin/env python
"""
Hiveary
https://hiveary.com

Licensed under Simplified BSD License (see LICENSE)
(C) Hiveary, Inc. 2013-2014 all rights reserved
"""

import glob
import impala
import importlib
import inspect
import json
import logging
import os
import signal
import socket
import sys
from twisted.internet import reactor, task

# esky is only needed for updating the application if its frozen
if hasattr(sys, 'frozen'):
  import esky

# Local imports
from . import __version__
from . import daemon
from . import monitors
from . import network
from . import paths
import hiveary.info.system


class RealityAuditor(daemon.Daemon):
  """Daemon subclass. Makes sure that data is being collected and aggregated,
  network connections are active, and gravity is working."""

  INITIAL_DELAY = 5  # Small delay to make sure the network has been initialized
  UPDATE_TIMER = 60 * 60 * 8  # How often to check for agent updates, in seconds
  PID_FILE = '/var/run/hiveary-agent.pid'  # Default location of the PID file
  REMOTE_HOST = 'hiveary.com'  # Default server to connect to
  MONITORS_DIR = '/usr/lib/hiveary/'  # Default location to find monitor modules

  def __init__(self, parsed_args, stored_config, logger=None):
    """Initialization when the agent is started.

    Args:
      parsed_args: A dictionary version of the parsed command line arguments.
      stored_config: A dictionary of any values from the config file.
      logger: Optional, the logger that this class should log to.
    """

    self.logger = logger or logging.getLogger('hiveary_agent.controller')

    # Check that the directory for the pid file exists, and if not then use
    # the same directory as the config file
    pid_file = stored_config.get('pid_file', self.PID_FILE)
    if not os.path.isdir(os.path.dirname(pid_file)):
      directory = os.path.dirname(stored_config['filename'])
      filename = os.path.basename(pid_file)
      pid_file = os.path.join(directory, filename)

    # Check if the monitors directory exists, otherwise, place it under the
    # location of the config file
    self.monitors_dir = stored_config.get('monitors_dir') or self.MONITORS_DIR
    if not os.path.isdir(self.monitors_dir):
      directory = os.path.dirname(stored_config['filename'])
      self.monitors_dir = os.path.join(directory, 'monitors')
      if not os.path.isdir(self.monitors_dir):
        os.makedirs(self.monitors_dir)

    self.monitors = []

    # Get and possibly save optional configuration parameters. If the defaults
    # are used, they won't be saved.
    self.extra_options = {}
    for option in ('monitor_backoff', 'pid_file', 'ca_bundle', 'monitors_dir'):
      value = stored_config.get(option)
      if value:
        self.extra_options[option] = value

    # Network controller is initalized by the agent with the necessary
    # authentication credentials
    self.network_controller = network.NetworkController(reactor)
    self.set_config(parsed_args, stored_config)

    # Setup a handler to interpret interrupts since twisted overrides them
    signal.signal(signal.SIGINT, self.signal_handler)

    executable, args = paths.find_executable()
    self.logger.debug('Setting daemon to use %s %s', executable, args)
    super(RealityAuditor, self).__init__(pid_file, executable=executable,
                                         args=args)

  def run(self):
    """Called once the agent has been daemonized, or if the agent is running
    in the foreground. All monitors are started from here and communication
    with the server is started."""

    self.network_controller.ensure_internet_connection()

    if hasattr(sys, 'frozen'):
      # Setup the auto-updater
      update_path = 'https://{host}/versions'.format(host=self.network_controller.remote_host)
      self.app = esky.Esky(sys.executable, update_path)
      reactor.callLater(0, self.start_loop, self.UPDATE_TIMER,
                        self.auto_agent_update)

    # Load all monitors
    self.load_monitors()

    # Connect to the server
    self.network_controller.initialize_amqp()

    # Start all of our monitors
    for monitor in self.monitors:
      self.start_monitor(monitor)

    # Send the first data dump
    data = hiveary.info.system.pull_all()
    data['version'] = __version__
    data['host_id'] = self.network_controller.obj_id
    data['monitors'] = []
    for monitor in self.monitors:
      monitor_data = {
          'name': monitor.NAME,
          'sources': monitor.SOURCES,
          'id': monitor.UID,
          'type': monitor.TYPE,
      }
      data['monitors'].append(monitor_data)
    reactor.callLater(self.INITIAL_DELAY,
                      self.network_controller.publish_info_message,
                      'startup',
                      json.dumps(data))

    # Send a ping to the server to act as a keep-alive.
    reactor.callLater(self.INITIAL_DELAY, self.start_loop,
                      self.network_controller.PING_TIMER,
                      self.network_controller.ping_pong)

    reactor.run()

  def load_monitors(self):
    """Loads all monitors from the config file. If it cannot find a configured module,
    it will attempt to download one.
    """

    # Check that we have monitors enabled in the config, and know where to find them
    if self.monitors_dir:
      # Add the monitors dir to path so we can import monitors
      self.logger.info('Loading monitors from %s', self.monitors_dir)
      sys.path.insert(0, self.monitors_dir)

      # When frozen, the monitors included with the current version also
      # need to be loaded
      if hasattr(sys, 'frozen'):
        frozen_monitors_dir = os.path.join(os.path.dirname(sys.executable),
                                           'monitors')
        sys.path.insert(0, frozen_monitors_dir)
        self.logger.debug('Also loading frozen monitors from: %s',
                          frozen_monitors_dir)

      # Find all of the monitor modules.
      monitor_map = {}
      for filename in glob.glob('%s/*.py' % self.monitors_dir):
        monitor = os.path.basename(filename).split('.')[0]
        monitor_map['hiveary.monitors.%s' % monitor] = filename

      # Import all of the monitors and add the instances to our monitor list.
      impala.register(monitor_map)
      object_filter = lambda obj: inspect.isclass(obj) and monitors.BaseMonitor in inspect.getmro(obj)
      for module_name in monitor_map.keys():
        try:
          module = importlib.import_module(module_name)
        except ImportError:
          self.logger.error('Failed to import module %s', module_name)
          continue

        # Filter down to all classes that inherit the monitors.BaseMonitor class.
        for class_name, monitor_class in inspect.getmembers(module, object_filter):
          self.logger.info('Loading %s from %s', class_name, module_name)
          try:
            monitor = monitor_class()
          except Exception:
            self.logger.error('Failed to load class %s from module %s',
                              class_name, module_name)
            self.logger.debug('Full loading error:', exc_info=True)
          else:
            self.monitors.append(monitor)

  def start_monitor(self, monitor):
    """Starts a given monitor.

    Args:
      monitor: Instance of a monitor class
    """

    self.logger.debug('Starting %s (%s) monitor data checks', monitor.NAME,
                      monitor.UID)
    self.network_controller.monitors[monitor.UID] = monitor
    monitor.send_alert = self.network_controller.publish_alert_message

    # Check if the monitor should run in a loop
    if monitor.MONITOR_TIMER is not None:
      self.start_loop(monitor.MONITOR_TIMER, monitor.run)
    else:
      reactor.callInThread(monitor.run)

    self.start_aggregation_loop(monitor)

  def signal_handler(self, signum, stackframe):
    """Handles a SIGTERM or SIGINT sent to the process.

    Args:
      signum: The number of the signal. SIGINT is 2, SIGTERM is 15.
      stackframe: The interrupted stack frame.
    """

    self.logger.info("Received signal: %s", signum)
    self.shutdown()

  def shutdown(self):
    """Stops the agent and any active network connections."""

    # Clean up the daemon after the reactor is done
    reactor.addSystemEventTrigger('after', 'shutdown', self.delpid)

    self.network_controller.stop_amqp()
    reactor.stop()

  def set_config(self, args, stored_config):
    """Update any configuration variables. Use passed command-line values first,
    then stored config values, then defaults.

    Args:
      args: A dictionary version of the parsed command line arguments.
      stored_config: A dictionary of any values from the config file.
    """

    stored_config.update((k, v) for k, v in args.iteritems() if v is not None)
    self.logger.debug('Using the merged config: %s', stored_config)

    # Store the required network config values in the network controller
    self.network_controller.debug_mode = stored_config['debug']
    self.network_controller.disable_ssl_verification = stored_config['disable_ssl_verify']
    self.network_controller.hostname = socket.getfqdn()
    self.network_controller.owner = stored_config.get('username')
    self.network_controller.access_token = stored_config.get('access_token')
    self.network_controller.remote_host = stored_config.get('server') or self.REMOTE_HOST
    self.network_controller.amqp_server = stored_config.get('amqp_server') or 'amqp.{domain}'.format(
        domain=self.network_controller.remote_host)
    self.network_controller.ca_bundle = stored_config.get('ca_bundle')

    # Update the config file if needed
    if args.get('update'):
      self.update_config_file(stored_config['filename'])

  def update_config_file(self, filename):
    """Stores the current configuration in the config file, overwriting current
    values.

    Args:
      filename: The full path to the config file.
    """

    config = {
        'server': self.network_controller.remote_host,
        'access_token': self.network_controller.access_token,
        'username': self.network_controller.owner,
        'amqp_server': self.network_controller.amqp_server,
        'monitors': self.monitor_config,
    }

    config.update(self.extra_options)

    with open(filename, 'w') as file_desc:
      json.dump(config, file_desc, indent=2)

  def start_aggregation_loop(self, monitor):
    """Starts the aggregation loop, setting it up to use the exact intervals
    if the passed monitor uses intervals.

    Args:
      monitor: The monitor object. Must be a subclass of BaseMonitor.
    """

    if hasattr(monitor, 'next_interval'):
      # Only start aggregating data at a specified time to make sure that we
      # maintain the same time intervals. The first run will contain a partial
      # period if we have data points from at least half that period.
      delta = monitor.next_interval()
      self.logger.debug('Starting %s aggregation loop in %s seconds', monitor.NAME, delta)
    else:
      delta = monitor.AGGREGATION_TIMER

    reactor.callLater(int(delta), self.start_loop,
                      monitor.AGGREGATION_TIMER,
                      monitor.send_data, self.network_controller)

  def start_loop(self, timer, func, *args, **kwargs):
    """Continuously loop the passed function.

    Args:
      timer: How often to loop the call, in seconds.
      func: The function to run.
      *args, **kwargs: Anything that needs to be passed to the function.
    """

    loop = task.LoopingCall(func, *args, **kwargs)
    deferred_task = loop.start(timer, now=True)
    deferred_task.addErrback(self.logger.error)

  def restart(self):
    """Setup an event to restart the agent after the reactor stops."""

    self.logger.warn('The agent is restarting...')

    reactor.addSystemEventTrigger('after', 'shutdown', self.fork,
                                  detached=False, exit=False)
    self.shutdown()

  def auto_agent_update(self):
    """Checks for a new version of the running application from the remote server.
    If a new version is found, it will be downloaded and extracted, and the agent
    restarted. This only applies if the application is frozen."""

    self.logger.info('Checking for updates, currently running %s',
                     self.app.active_version)

    new_version = self.app.find_update()
    if new_version:
      self.logger.info('Version %s found', new_version)

      self.app.auto_update(callback=self.logger.debug)
      self.logger.info('New version installed')

      self.restart()
    else:
      self.logger.info('No update found')



#!/usr/bin/env python
# coding=utf8

# Copyright (C) 2010 Saúl ibarra Corretgé <saghul@gmail.com>
#
#

"""
Modified from: https://gist.github.com/saghul/542780

Hiveary
https://hiveary.com

Licensed under Simplified BSD License (see LICENSE)
(C) Hiveary, Inc. 2014 all rights reserved

pydmesg: dmesg with human-readable timestamps
"""

import re
import subprocess
import sys


# Regex for dmesg lines that allows the source of the message to be obtained.
_dmesg_line_regex = re.compile("^\[.*?\] ((?P<source>.*?): )?.*$")


def exec_process(cmdline, silent, input=None, **kwargs):
  """Execute a subprocess and returns the returncode, stdout buffer and stderr buffer.
     Optionally prints stdout and stderr while running."""
  try:
    sub = subprocess.Popen(cmdline, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, **kwargs)
    stdout, stderr = sub.communicate(input=input)
    returncode = sub.returncode
    if not silent:
      sys.stdout.write(stdout)
      sys.stderr.write(stderr)
  except OSError, e:
    if e.errno == 2:
      raise RuntimeError('"%s" is not present on this system' % cmdline[0])
    else:
      raise
  if returncode != 0:
    raise RuntimeError('Got return value %d while executing "%s", stderr output was:\n%s' % (returncode, " ".join(cmdline), stderr.rstrip("\n")))
  return stdout


def human_dmesg(source=None, max_lines=20):
  """Format and return lines from the output of dmesg.

  Args:
    source: If provided, only lines from this source will be returned.
    max_lines: The maximum number of lines to return, in reverse order.
  Returns:
    None in the event of an error, otherwise an array of the last dmesg lines.
  """

  formatted_dmesg = []

  # Reversing the array shows the newest lines first, to stay consistent
  # with how other logs are presented.
  try:
    dmesg_data = exec_process(['dmesg', '-T'], True).split('\n')
    dmesg_data.reverse()
    for line in dmesg_data:
      if not line:
        continue

      if source:
        match = _dmesg_line_regex.match(line)
        if match:
          source_match = match.groupdict('UNKNOWN')['source']
          if source in source_match:
            formatted_dmesg.append(line)
      else:
        formatted_dmesg.append(line)

      if len(formatted_dmesg) >= max_lines:
        break
  except RuntimeError:
    return []
  else:
    return formatted_dmesg


if __name__ == '__main__':
  print human_dmesg()



#!/usr/bin/env python
"""
Hiveary
https://hiveary.com

Licensed under Simplified BSD License (see LICENSE)
(C) Hiveary, Inc. 2013-2014 all rights reserved

Functions for collecting local system information.
"""

import logging
import netifaces
import os
import platform
import psutil
import simplejson
import socket
import subprocess

if subprocess.mswindows:
  import win32net
  import win32service
  import wincom
else:
  import grp
  import pwd

import hiveary.info


logger = logging.getLogger('hiveary_agent.info.system')


def find_valid_disks():
  """Iterate through and find all valid disk partitions. Some devices will not
  be accessible since they have non-disk filesystems, such as a CD-ROM drive.

  Returns:
    A list of all valid disks.
  """

  disks = []
  for disk in psutil.disk_partitions():
    try:
      psutil.disk_usage(disk.device)
    except OSError:
      continue
    else:
      disks.append(disk.device)

  return disks


def pull_all():
  """Finds all available static information about the host.

  Returns:
    A dictionary containing all discovered information.
  """

  logger.info('Pulling all local info')
  info = {}

  info['os'], info['processor'] = pull_os()
  info['disks'] = pull_disks()
  info['fqdn'], info['interfaces'] = pull_net()
  info['users'] = pull_users()
  info['groups'] = pull_groups()
  info['services'] = pull_services()
  info['automatic_update_settings'] = pull_update_settings()

  return info


def pull_os():
  """Find information about the host's operating system.

  Returns:
    A tuple of a dictionary of the OS information, and the system's processor.

    Example:
    (
     {
       'architecture': 'nt',
        'platform': 'Windows',
        'release': {
          'product_name': 'Windows 8',
          'service_pack': 0,
          'version_number': '6.2'
      },
     'Intel64 Family 6 Model 58 Stepping 9, GenuineIntel'
    )
  """

  os_info = {}
  os_info['architecture'] = os.name
  os_info['platform'] = hiveary.info.current_system
  if hiveary.info.current_system == 'Windows':
    os_info['release'] = wincom.get_version_info()
  else:
    os_info['release'] = platform.release()

  processor = platform.processor()

  return (os_info, processor)


def pull_disks():
  """Finds full disk information from the host.

  Returns:
    A dictionary of all disk-like devices mapped to their data. Example:

    {
      'C:\\': {'filesystem': 'NTFS', 'mountpoint': 'C:\\', 'options': 'rw,fixed'},
      'D:\\': {'filesystem': 'NTFS', 'mountpoint': 'D:\\', 'options': 'rw,fixed'},
      'E:\\': {'filesystem': 'NTFS', 'mountpoint': 'E:\\', 'options': 'rw,fixed'},
      'H:\\': {'filesystem': '', 'mountpoint': 'H:\\', 'options': 'cdrom'}
    }
  """

  disks = {}
  for disk in psutil.disk_partitions():
    disks[disk.device] = {
        'mountpoint': disk.mountpoint,
        'filesystem': disk.fstype,
        'options': disk.opts,
    }
  return disks


def pull_net():
  """Finds network related information.

  Returns:
    A tuple of the host's FQDN and a list of network interfaces. Each network
    interface in the list is a dictionary of the interface properties. Example:

    (
      'prism7.utah.nsa.gov',
      [
        {
        'af_inet': {
          'addr': '127.0.0.1',
          'netmask': '255.0.0.0',
          'peer': '127.0.0.1'
        },
        'af_inet6': {
          'addr': '::1',
          'netmask': 'ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff'},
          'af_packet': {
            'addr': '00:00:00:00:00:00',
            'peer': '00:00:00:00:00:00'
          },
        'name': 'lo'
        },
        ...
      ]
    )
  """

  fqdn = socket.getfqdn()
  interfaces = []
  for iface in netifaces.interfaces():
    ifaddrs = netifaces.ifaddresses(iface)
    interface = {'name': iface}

    # Translate the address family identifier to a name
    for num, addrs in ifaddrs.iteritems():
      family = netifaces.address_families[num]

      # Parse out the netmask for IPv6
      if family == 'AF_INET6':
        addresses = []
        for address in addrs:
          addr = address.pop('addr')
          split_addr = addr.split('%')
          address['addr'] = split_addr[0]

          if len(split_addr) == 2:
            address['scope_id'] = split_addr[1]

          addresses.append(address)
      else:
        addresses = addrs

      if len(addresses) == 1:
        addresses = addresses[0]
      interface[family.lower()] = addresses

    interfaces.append(interface)

  logger.debug('Retrieved info on %s interfaces', len(interfaces))
  return (fqdn, interfaces)


def pull_users():
  """Finds local user information.

  Returns:
    A list of users. Each user is a dictionary of user data. Example:

    [
      {
        'gecos': 'root',
        'gid': 0,
        'homedir': '/root',
        'name': 'root',
        'shell': '/bin/bash',
        'uid': 0
      },
      ...
    ]
  """

  if hiveary.info.current_system == 'Windows':
    net_users = win32net.NetUserEnum(None, 3)
    logger.debug('Found %s users', len(net_users[0]))
    resume = net_users[2]

    for user in net_users[0]:
      del(user['logon_hours'])  # Hex isn't JSONable
    users = net_users[0]

    while resume != 0:
      net_users = win32net.NetUserEnum(None, 3, resumeHandle=resume)
      logger.debug('Found %s users', len(net_users[0]))
      resume = net_users[2]

      for user in net_users[0]:
        del(user['logon_hours'])  # Hex isn't JSONable
      users.append(net_users[0])
  elif hiveary.info.current_system == 'Linux' or hiveary.info.current_system == 'Darwin':
    # pwd/grp work on linux and osx
    users = []
    users_pwd = pwd.getpwall()
    for user_pwd in users_pwd:
      user = {
          'name': user_pwd.pw_name,
          'uid': user_pwd.pw_uid,
          'gid': user_pwd.pw_gid,
          'gecos': user_pwd.pw_gecos,
          'homedir': user_pwd.pw_dir,
          'shell': user_pwd.pw_shell
        }
      users.append(user)
  else:
    logger.error('Unkown platform "%s", cannot pull users', hiveary.info.current_system)
    return []

  return users


def pull_groups():
  """Finds local group information.

  Returns:
    A list of groups. Each group is a dictionary of information about the group.

    Example:
    [
      {
        'gid': 0,
        'members': ['root'],
        'name': 'root'
      },
      ...
    ]
  """

  if hiveary.info.current_system == 'Windows':
    net_local_groups = win32net.NetLocalGroupEnum(None, 1)
    logger.debug('Found %s groups', len(net_local_groups[0]))
    resume = net_local_groups[2]
    groups = net_local_groups[0]

    while resume != 0:
      net_local_groups = win32net.NetLocalGroupEnum(None, 1, resumeHandle=resume)
      logger.debug('Found %s groups', len(net_local_groups[0]))
      resume = net_local_groups[2]
      groups.extend(net_local_groups[0])

    # Enumerate group members
    for group in groups:
      net_members = win32net.NetLocalGroupGetMembers(None, group['name'], 1)
      resume = net_members[2]
      group['members'] = [x['name'] for x in net_members[0]]

      while resume != 0:
        net_members = win32net.NetLocalGroupGetMembers(None, group['name'], 1, resumeHandle=resume)
        resume = net_members[2]
        group['members'].extend([x['name'] for x in net_members[0]])
  elif hiveary.info.current_system == 'Linux' or hiveary.info.current_system == 'Darwin':
    groups = []
    groups_grp = grp.getgrall()
    for group_grp in groups_grp:
      group = {
          'name': group_grp.gr_name,
          'gid': group_grp.gr_gid,
          'members': group_grp.gr_mem
        }
      groups.append(group)
  else:
    logger.error('Unkown platform "%s", cannot pull groups', hiveary.info.current_system)
    return []

  return groups


def pull_services():
  """Finds information about installed services, currently only applicable to
  Windows.

  Returns:
    A dictionary of service name mapped to service information. All strings will
    be in unicode. The service information primarily includes the user-displayed
    service name and a dictionary of status information. Example:

    {
      u'wuauserv': {
        'display_name': u'Windows Update',
        'status': {
          'checkpoint': 0,
          'controls_accepted': 0,
          'service_exit_code': 0,
          'state': 1,
          'type': 32,
          'wait_hint': 0,
          'win32_exit_code': 0
        }
      },
      ...
    }
  """

  if hiveary.info.current_system == 'Windows':
    services = {}
    scm = win32service.OpenSCManager(
        None, None, win32service.SC_MANAGER_ENUMERATE_SERVICE)

    raw_services = win32service.EnumServicesStatus(
        scm, win32service.SERVICE_WIN32)
    scm.Close()

    # Parse the services into something useful
    for raw_service in raw_services:
      service = {}
      service['display_name'] = raw_service[1]

      # Parse the status flags into meaningful names
      raw_status = raw_service[2]
      status = {
          'type': raw_status[0],
          'state': raw_status[1],
          'controls_accepted': raw_status[2],
          'win32_exit_code': raw_status[3],
          'service_exit_code': raw_status[4],
          'checkpoint': raw_status[5],
          'wait_hint': raw_status[6]
        }

      service['status'] = status

      # Key the service by ServiceName
      services[raw_service[0]] = service
  else:
    # TODO: nix services and osx services
    logger.warn('Service enumeration not implemented for %s', hiveary.info.current_system)
    return {}

  logger.debug('Retrieved the services info')
  return services


def pull_processes(top=None, top_number=5):
  """Retrieves information about active processes.

  Args:
    top: If provided, the `top_number` of processes for the matching
        resource will be returned. Valid options are memory_percent or
        cpu_percent
    top_number: Number of top processes to return for the given resource. Only
        valid when top is not None.
  Returns:
    Returns a tuple of (all processes, top processes).
    Each process in the list is a dictionary with nicode keys containing
    a large amount of detailed information for the process. Example:

    [
      {
        u'cmdline': [],
        u'connections': [
          {
            u'family': 23,
            u'fd': -1,
            u'laddr': [u'::1', 8888],
            u'raddr': [u'::1', 52912],
            u'status': u'TIME_WAIT',
            u'type': 1
          },
          ...
        ],
        u'cpu_affinity': None,
        u'cpu_percent': 0.0,
        u'cpu_times': {u'system': 526099.53125, u'user': 0.0},
        u'create_time': 1381682351.719,
        u'cwd': None,
        u'exe': None,
        u'ext_memory_info': {
          u'nonpaged_pool': 0,
          u'num_page_faults': 0,
          u'paged_pool': 0,
          u'pagefile': 0,
          u'peak_nonpaged_pool': 0,
          u'peak_paged_pool': 0,
          u'peak_pagefile': 0,
          u'peak_wset': 0,
          u'private': 0,
          u'wset': 20480
        },
        u'io_counters': {
          u'read_bytes': 0,
          u'read_count': 0,
          u'write_bytes': 0,
          u'write_count': 0
        },
        u'ionice': None,
        u'memory_info': {u'rss': 20480, u'vms': 0},
        u'memory_maps': None,
        u'memory_percent': 0.00023943963463427274,
        u'name': u'System Idle Process',
        u'nice': None,
        u'num_ctx_switches': {u'involuntary': 0, u'voluntary': 703011786},
        u'num_handles': 0,
        u'num_threads': 4,
        u'open_files': [],
        u'pid': 0,
        u'ppid': 0,
        u'status': u'running',
        u'threads': None,
        u'username': u'NT AUTHORITY\\SYSTEM'
      },
      ...
    ]
  """

  # Create a JSONable version of the process list
  processes = []
  for p in psutil.process_iter():
    try:
      processes.append(simplejson.loads(simplejson.dumps(p.as_dict())))
    except psutil.NoSuchProcess:
      # Likely poor timing for a terminating process, not worth logging
      continue
    except OSError:
      if p.name == 'System':
        # System cannot be converted to a dict, just ignore it
        continue
      else:
        raise

  # Find the top processes
  top_procs = []
  if top:
    full_top_procs = sorted(processes, key=lambda p: p[top], reverse=True)
    top_procs = full_top_procs[:top_number]

  logger.debug('Retrieved the running processes')
  return processes, top_procs


def pull_update_settings():
  """Retrieves information about the system's auto update settings, currently
  only applicable to Windows.

  Retruns:
    A dictionary of the update settings. Example:

    {
      'service_enabled': True,
      'notification_level': 'Scheduled Installation',
      'installation_schedule': {'day': 'Every Day', 'time': '03:00'}
    }
  """

  logger.debug('Retrieving system update settings.')

  if hiveary.info.current_system == 'Windows':
    automatic_update_settings = wincom.get_update_settings()
    logger.debug('Retrieved the following update settings: %s',
                 automatic_update_settings)
  else:
    automatic_update_settings = {}

  return automatic_update_settings



#!/usr/bin/env python
"""
Hiveary
https://hiveary.com

Licensed under Simplified BSD License (see LICENSE)
(C) Hiveary, Inc. 2014 all rights reserved

Functions for collecting local log information.
"""

import datetime
import logging
import os
import subprocess

if subprocess.mswindows:
  import win32evtlog
  import win32evtlogutil
  import win32con
  import winerror
else:
  import hiveary.info.dmesg

import hiveary.info

NUM_LOG_LINES = 20
logger = logging.getLogger('hiveary_agent.info.logs')


def log_files(process, fuzzy_matches=None):
  """Finds the log file being used by the passed process.

  Args:
    process: A psutil.Process instance in dict form.
    fuzzy_matches: An optional iterable of strings to compare against the
        process's open files.
  Returns:
    The full path to the process's log file, or None if it could not
    be determined.
  """

  fuzzy_matches = fuzzy_matches or ('log', 'err', 'info')
  log_paths = []

  for open_file in process.get('open_files', []):
    path = open_file['path']

    # Ignore duplicated file handlers.
    if path in log_paths:
      continue

    # Ignore any binary files.
    if 'text' not in subprocess.check_output(['file', '-b', path]):
      continue

    # Anything within a log directory can be included, regardless of name
    if '/log/' in path:
      log_paths.append(path)
      continue

    filename = os.path.basename(path)

    for fuzzy_match in fuzzy_matches:
      if fuzzy_match in filename:
        log_paths.append(path)
        break

  return log_paths


def tail_file(filename, num_lines=NUM_LOG_LINES):
  """Pulls the last lines from a file.

  Args:
    filename: The name of the file to pull from.
    num_lines: The number of lines to read from the end of the file.
  Returns:
    A string of the lines from the file, or None.
  """

  try:
    # stderr is piped to stdout so that if an error occurs it isn't
    # propogated up to the parent process.
    file_lines = subprocess.check_output(['tail', '-n', str(num_lines), filename],
                                         stderr=subprocess.STDOUT)
  except subprocess.CalledProcessError as err:
    logger.error('Unable to read %s - returned %d: %s',
                 filename, err.returncode, err.message)
  else:
    return file_lines


def read_event_log(logtype='Application', max_entries=NUM_LOG_LINES, hours_back=12):
  """Reads events from the Windows event log.

  Args:
    logtype: The event log to read. One of System, Security, Application.
    max_entries: The maximum number of entries to return from the event log.
    hours_back: How many hours to go back into the logs. Events older than this
        will not be returned.
  Returns:
    An array of recent events from the event log.
  """

  try:
    handle = win32evtlog.OpenEventLog('127.0.0.1', logtype)
  except win32evtlog.error:
    logger.error('Unable to access the %s event log', logtype, exc_info=True)
    return

  # This dict converts the event type into a human readable form. Taken from:
  # http://timgolden.me.uk/pywin32-docs/Windows_NT_Eventlog.html
  evt_types = {
      win32con.EVENTLOG_AUDIT_FAILURE: 'AUDIT_FAILURE',
      win32con.EVENTLOG_AUDIT_SUCCESS: 'AUDIT_SUCCESS',
      win32con.EVENTLOG_INFORMATION_TYPE: 'INFORMATION',
      win32con.EVENTLOG_WARNING_TYPE: 'WARNING',
      win32con.EVENTLOG_ERROR_TYPE: 'ERROR',
  }
  flags = win32evtlog.EVENTLOG_BACKWARDS_READ | win32evtlog.EVENTLOG_SEQUENTIAL_READ

  relevant_events = []
  oldest = datetime.datetime.now() - datetime.timedelta(hours=hours_back)

  while True:
    events = win32evtlog.ReadEventLog(handle, flags, 0)
    if not events:
      break

    for event in events:
      # Limit the total amount of messages retrieved.
      if len(relevant_events) >= max_entries:
        return relevant_events

      # Only send recent events as events that are too old are unlikely to be
      # useful.
      time_gen = datetime.datetime.strptime(str(event.TimeGenerated),
                                            '%m/%d/%y %H:%M:%S')
      if time_gen < oldest:
        return relevant_events

      formatted_event = {
          'Event ID': winerror.HRESULT_CODE(event.EventID),
          'Time': event.TimeGenerated.Format(),
          'Source': str(event.SourceName),
          'Message': win32evtlogutil.SafeFormatMessage(event, logtype),
          'Type': evt_types.get(event.EventType),
      }
      relevant_events.append(formatted_event)

  return relevant_events


def read_syslog(num_lines=NUM_LOG_LINES):
  """Reads the last lines that were sent to syslog.

  Returns:
    An array of lines from the end of the syslog file, or None.
  """

  possible_syslog_files = ('/var/log/syslog', '/var/log/messages')

  for filename in possible_syslog_files:
    if os.path.isfile(filename):
      break
  else:
    logger.info('A syslog file could not be found on this system.')
    return

  syslog_lines = tail_file(filename)
  if syslog_lines:
    syslog_lines = syslog_lines.strip().split('\n')
    syslog_lines.reverse()
    return syslog_lines


def read_system_logs():
  """Platform-agnostic function for reading the most recent log records from the
  system.

  Returns:
    A dictionary mapping the name or path of the system log to its most recent
    entries.
  """

  system_logs = {}

  if hiveary.info.current_system == 'Windows':
    for event_log_source in ('Application', 'System', 'Security'):
      system_logs[event_log_source] = read_event_log(logtype=event_log_source)
  elif hiveary.info.current_system == 'Linux':
    dmesg_lines = hiveary.info.dmesg.human_dmesg()
    if dmesg_lines:
      system_logs['dmesg'] = dmesg_lines

    syslog_lines = read_syslog()
    if syslog_lines:
      system_logs['syslog'] = system_logs

  return system_logs



#!/usr/bin/env python
"""
Hiveary
https://hiveary.com

Licensed under Simplified BSD License (see LICENSE)
(C) Hiveary, Inc. 2014 all rights reserved

Functions for collecting local system information.
"""

import platform

current_system = platform.system()
